/** Catalogue.java*/
package fournisseurs;
import java.util.*;
import java.io.*;
import parcInfo.*;


@SuppressWarnings("serial")
public class Catalogue implements Serializable
{
	protected TreeMap<String,Produits> listeProduits;
	private String nom;
	
	/**
	 * @param listeProduits
	 * @param nom
	 */
	public Catalogue(String nom) {
		this.nom = nom;
		this.listeProduits = new TreeMap<String,Produits> ();
	}
	
	/**
	 * @return le nom
	 */
	public String getNom() {
		return nom;
	}
	
	/**
	 * @param nom  initialise la variable nom
	 */
	public void setNom(String nom) {
		this.nom = nom;
	}
	
	/**
	 * @param code
	 * @return le produit associe au nom
	 */
	public Produits getProduit(String code) {
		return listeProduits.get(code);
	}
	
	/**
	 * @return la liste des produits
	 */
	public TreeMap<String,Produits> getListeProduit() {
		return this.listeProduits;
	}
	
	/**
	 * @param produit
	 * Ajoute un produit au catalogue
	 */
	public void addProduit(Produits p)
	{
		listeProduits.put(p.getReference(),p);
	}
	
	/**
	 * Supprime le produit du Catalogue 
	 * @param p : Produit a supprimer
	 */
	public void delProduit(Produits p)
	{
		try{
			listeProduits.remove(p);
		}
		catch(ClassCastException ex){};
		
	}
	
	/**
	 * Supprime tous les Produits d'un catalogue
	 * @since 1.5
	 */
	public void delAllProduit()
	{
		for (String code :listeProduits.keySet())
			listeProduits.remove(listeProduits.get(code));
	}
	
	
	/**
	 * fonction qui retourne le Meilleur prix du produit du catalogue 
	 * si elle le trouve -1 sinon
	 * @param e Equipement
	 * @return float Meilleur Prix
	 * @since 1.5  
	 */
	public float getMeilleurPrixCat(Equipement e)
	{
		float prixCat=1000000000;
		Produits p;
		OrdinateurCat o;
		ImprimantesCat i;
		for (String code:listeProduits.keySet())
		{
			p=listeProduits.get(code) ;
			if(p instanceof ImprimantesCat)
			{
				i=(ImprimantesCat)p; 
				if(i.egalOuSuperieur(e))
				{
					if(prixCat>i.getPrix())
						prixCat=i.getPrix();
				}
			}
			
			if(p instanceof OrdinateurCat)
			{
				o=(OrdinateurCat)p;
				if(o.egalOuSuperieur(e))
				{
					// On recalcule le prix a partir des composants de l'ordinateur
					o.setPrix(o.calculPrixOrdinateur());
					if(prixCat>o.getPrix())
						prixCat=o.getPrix();
				}
			}
			
		}
		
		if (prixCat!=1000000000)
			return prixCat;
		
		return -1;
	}
	
	/**
	 * @param Composant c : Le composant a chercher
	 * @return float Meilleur prix du composant dans le catalogue
	 * ou -1 si non trouve
	 * @since 1.5
	 */
	public float getMeilleurPrixCat(Composants c)
	{
		// On ne compare Ici Que les composants seul
		float prixCat=1000000000;
		Produits p;
		for (String code:listeProduits.keySet())
		{
			p=listeProduits.get(code) ;
			if(p.egalOuSuperieur(c,true))
			{
				if(prixCat>p.getPrix())
					prixCat=p.getPrix();
			}
		}
		
		if (prixCat!=1000000000)
			return prixCat;
		
		return -1;
	}
		
	/**
	 * Methode permettant d'afficher les caracteristiques
	 * du catalogue
	 * @return String
	 */
	public String toString()
	{
		return "nom :"+ this.nom;
	} 
	
}
