/** Fournisseur.java */
package fournisseurs;
import parcInfo.*;
import java.util.*;
import java.io.*;

@SuppressWarnings("serial")
public class Fournisseur implements Serializable
{
	private String nom;      
	public TreeMap<String,Catalogue>listeCatalogues;                       
	
	/**
	 * @param nom
	 */
	public Fournisseur(String nom) {
		super();
		this.nom = nom;
		this.listeCatalogues = new TreeMap<String,Catalogue> ();
	}
	
	/**
	 * @return le nom
	 */
	public String getNom() {
		return nom;
	}
	
	/**
	 * @param nom initialise la variable nom
	 */
	public void setNom(String nom) {
		this.nom = nom;
	}
	
	/**
	 * @param code : nom du catalogue a rechercher
	 * @return Catalogue le catalogue associe
	 */
	public Catalogue getCatalogue(String code) {
		return this.listeCatalogues.get(code);
	}
	
	/**
	 * Ajoute un catalogue au fournisseur
	 * @param c Catalogue a ajouter
	 */
	public void addCatalogue(Catalogue c){
		listeCatalogues.put(c.getNom(),c);		
	}
	
	/**
	 * Supprime le catalogue passe en parametre
	 * @param c catalogue a supprimer
	 */
	public void delCatalogue(Catalogue c){
		listeCatalogues.remove(c);
	}
	
	/**
	 * Supprime tous les catalogues du fournisseur
	 * @since 1.5
	 */
	public void delAllCatalogue(){
		for (String code:listeCatalogues.keySet()){
			(listeCatalogues.get(code)).delAllProduit();
			listeCatalogues.remove(listeCatalogues.get(code));
		}
	}
	
	/**
	 * @return  la liste des catalogues
	 */
	public TreeMap<String,Catalogue> getListeCatalogues(){
		return this.listeCatalogues;
	}
	
	/**
	 * fonction qui retourne le Meilleur prix de l'equipement  
	 * sur tous les catalogues du fournisseur si elle le trouve -1 sinon
	 * @param e Equipement
	 * @return float Meilleur Prix
	 * @since 1.5  
	 */
	public float getMeilleurPrix(Equipement e)
	{
		float meilleurPrix=10000000;
		float prix;
		
		for(String code : listeCatalogues.keySet()){
			prix=listeCatalogues.get(code).getMeilleurPrixCat(e);
			if(prix!=-1 && prix<meilleurPrix)
				meilleurPrix=prix;
		}
		if(meilleurPrix!=10000000)
			return meilleurPrix;
		return -1;
	}
	
	/**
	 * @param Composant c : Le composant a chercher
	 * @return float Meilleur prix du composant dans tous les catalogues
	 * du fournisseur ou -1 si non trouve
	 * @since 1.5
	 */
	public float getMeilleurPrix(Composants c)
	{
		float meilleurPrix=10000000;
		float prix;
		
		for(String code : listeCatalogues.keySet()){
			prix=listeCatalogues.get(code).getMeilleurPrixCat(c);
			if(prix!=-1 && prix<meilleurPrix)
				meilleurPrix=prix;
		}
		if(meilleurPrix!=10000000)
			return meilleurPrix;
		return -1;
	}
}
