/** OrdinateurCat.java */
package fournisseurs;
import java.util.*;
import parcInfo.*;

@SuppressWarnings("serial")
public class OrdinateurCat extends Produits
{
	private boolean estPortable;
	public TreeMap<String,ComposantsCat>listeComposants;
	
	/**
	 * @param marque
	 * @param prix
	 * @param reference
	 * @param type
	 * @param estPortable
	 * @param listeComposants
	 */
	public OrdinateurCat(String marque, float prix,String reference, String type, boolean estPortable) {
		super(marque, prix, reference, type);
		this.estPortable = estPortable;
		this.listeComposants = new TreeMap<String,ComposantsCat> ();
	}
	
	/**
	 * @return vrai si l'ordinateur est Portable
	 */
	public boolean isEstPortable() {
		return estPortable;
	}
	
	/**
	 * @param estPortable  initialise la variable estPortable
	 */
	public void setEstPortable(boolean estPortable) {
		this.estPortable = estPortable;
	}
	
	/**
	 * Ajoute le composantsCat a l'ordinateur
	 * @param c ComposantsCat a ajouter
	 */
	public void addComposant(ComposantsCat c)
	{
		listeComposants.put(c.getReference(), c);
	}
	
	/**
	 * Supprime le composantsCat a l'ordinateur
	 * @param c ComposantsCat a supprimer
	 */
	public void delComposant(ComposantsCat c)
	{
		listeComposants.remove(c);
	}
	
	/**
	 * Supprime tous les composantsCat d'un ordinateur
	 * @since 1.5
	 */
	public void delAllComposant()
	{
		ComposantsCat c;
		for (String code:listeComposants.keySet())
		{
			c=listeComposants.get(code);
			listeComposants.remove(c);	
		}
	}
	
	/**
	 * Affiche la liste des composantsCat d'un ordinateur
	 * @since 1.5
	 */
	public void listerConfiguration()
	{
		System.out.println(this);
		for(String ref : listeComposants.keySet()){
			System.out.println(listeComposants.get(ref).toString());
		}
	}
	
	/**
	 * Calcule avec la somme des prix de tous les composants 
	 * le prix de l'ordinateur
	 * @return float le prix de l'ordinateur
	 * @since 1.5
	 */
	public float calculPrixOrdinateur()
	{
		float prix=0;
		for(String ref : listeComposants.keySet()){
			prix+=listeComposants.get(ref).getPrix();
		}
		return prix;
	}
	
	/**
	 * Methode permettant d'afficher les caracteristiques
	 * du produit
	 * @return String
	 * @see fournisseurs.Produits#toString()
	 */
	public String toString()
	{
		String chaine="Ordinateur";
		
		if(estPortable)
			chaine=chaine+" portable";
		
		return chaine+ "\n"+super.toString()+"\n";
	}
	
	
	
	/**
	 * Compare un ordinateur et un composant
	 * @param Composants c : Composant a comparer
	 * @param boolean VenteAuDetail : vrai si le composant est
	 * vendu au detail
	 * @return boolean toujours a faux car un ordinateur n'est
	 * pas un composant
	 */
	public boolean egalOuSuperieur(Composants c, boolean VenteAuDetail)
	{
		return false;
	}
	
	/**
	 * Compare pour tous les composants de la liste passee en parametre si
	 * tous les composants sont egaux ou superieurs aux composants de l'ordinateurCat 
	 * @param listeComposantOrdinateur
	 * @return boolean Vrai si toute la liste est respectee faux sinon
	 * @since 1.5
	 */
	public boolean egalOuSuperieur(TreeMap<String,Composants>listeComposantOrdinateur){
		boolean trouve=true;
		boolean trouveComposant;
		
		Composants c;
		ComposantsCat cCat;
		
		for(String composants: listeComposantOrdinateur.keySet()){
			c=listeComposantOrdinateur.get(composants);
			trouveComposant=false;
			for (String composantsCat:this.listeComposants.keySet())
			{
				
				cCat=listeComposants.get(composantsCat);
				if (cCat!=null){
					// Peu importe si le composant est vendu au detail ou non
					trouveComposant=trouveComposant || (cCat.egalOuSuperieur(c,true)||cCat.egalOuSuperieur(c,false));
					
				}
			}
			if (!trouveComposant)
				trouve=false;
		}
		return trouve;
	}
	
	
	/** 
	 * @param Equipement e Equipement du parc
	 * On compare si pour tous les composants un a un
	 * si l'ordinateur est meilleur
	 * @see fournisseurs.Produits#egalOuSuperieur(parcInfo.Equipement)
	 */
	public boolean egalOuSuperieur(Equipement e)
	{
		boolean trouve=false;
		if (e instanceof Ordinateur && this.isEstPortable()==((Ordinateur)e).isEstPortable())
		{
			TreeMap<String,Composants> listeComposantOrdinateur=((Ordinateur)e).getListeComposant();
			
			trouve=this.egalOuSuperieur(listeComposantOrdinateur);    			
			return trouve;
		}    
		return false;
	}
}
