/** Ordinateur.java */
package parcInfo;
import java.util.*;

@SuppressWarnings("serial")
public class Ordinateur extends Equipement
{
	private int nbConnexions;
	private boolean estPortable;
	protected TreeMap<String,Composants>listeComposants;
	
	/**
	 * @param nom
	 * @param marque
	 * @param type
	 * @param nbConnexions
	 * @param estPortable
	 * @param listeComposants
	 */
	public Ordinateur(String nom, String marque, String type, boolean estPortable) {
		super(nom, marque, type);
		this.nbConnexions = 0;
		this.estPortable = estPortable;
		this.listeComposants = new TreeMap<String,Composants> ();
	}
	
	/**
	 * @return vrai si l'ordinateur est Portable
	 */
	public boolean isEstPortable() {
		return estPortable;
	}
	
	/**
	 * @param estPortable  initialise la variable estPortable
	 */
	public void setEstPortable(boolean estPortable) {
		this.estPortable = estPortable;
	}
	
	/**
	 * @return le nbre de Connexions
	 */
	public int getNbConnexions() {
		return nbConnexions;
	}
	
	/**
	 * @param nbConnexions  initialise la variable nbConnexions
	 */
	public void setNbConnexions(int nbConnexions) {
		this.nbConnexions = nbConnexions;
	}
	
	/**
	 * Incremente le nombre de connexions d'un ordinateur
	 *
	 */
	public void incrementerConnexion()
	{
		this.nbConnexions++;
	}
	
	/**
	 * Parcours tous les composants d'un ordinateur et cree une liste
	 * de tous les composants defectueux d'un Ordinateur et la retourne
	 * @return liste de composants Defectueux
	 * @since 1.5
	 */
	public TreeMap<String,Composants> getListeDefectueux()
	{
		TreeMap<String,Composants> tableauDefectueux=new TreeMap<String,Composants>();
		for(String code: listeComposants.keySet()){
			if (listeComposants.get(code).isEtat())
				tableauDefectueux.put(code,(listeComposants.get(code)));
		}
		return tableauDefectueux;
	}
	
	/**
	 * Renvoie la chaine de tous les composants Defecteueux
	 * @return chaine de caracteres contenant liste de Composants defectueux
	 */
	public String listerComposantsDefectueux()
	{
		String chaine="";
		for(String code: listeComposants.keySet()){
			if (listeComposants.get(code).isEtat())
				chaine +="\n\n"+listeComposants.get(code);
		}
		return chaine;
	}
	
	/**
	 * Parcours la liste des composants et retourne
	 * l'etat de l'ordinateur
	 * Des qu'un composant est defecteux, l'ordinateur est
	 * defectueux
	 * @return boolean vrai si defecteux faux sinon
	 * @since 1.5
	 */
	public boolean CalculerEtatOrdinateur()
	{
		for(String code: listeComposants.keySet()){
			if (listeComposants.get(code).isEtat())
				return true;
		}
		return false;
		
	}
	
	/**
	 * Ajoute un composant a l'ordinateur 
	 * @param c : Composant a ajouter
	 */
	public void addComposant(Composants c)
	{
		listeComposants.put(c.getNomComposant(),c);
	}
	
	/**
	 * Supprime le Composant decrit par son nom donne de l'ordinateur
	 * @param code : nom du composant
	 */
	public void delComposant(String code)
	{
		this.listeComposants.remove(code);
	}
	
	/**
	 * Supprime tous les composants d'un ordinateur
	 * @since 1.5
	 */
	public void delAllComposant()
	{
		for(String code: listeComposants.keySet())
			listeComposants.remove(code);
	}
	
	/**
	 * Retourne la liste des composants de l'ordinateur
	 * @return liste des Composants
	 */
	public TreeMap<String,Composants> getListeComposant()
	{
		return this.listeComposants;
		
	}
	
	/**
	 * Retourne le Composant decrit par son nom
	 * @param code : Nom du composant
	 * @return Composant
	 */
	public Composants getComposant(String code){
		return listeComposants.get(code);
	}
	
	/**
	 * Retourne la chaine contenant la configuration
	 * de l'ordinateur
	 * @return String
	 * @since 1.5
	 */
	public String listerConfiguration()
	{
		String chaine=this.toString();
		
		for(String code: listeComposants.keySet())
			chaine+="\n\n"+listeComposants.get(code);
		return chaine;	
	}
	
	
	/**
	 * Retourne la description de l'ordinateur
	 * @return String
	 * @see Equipement#toString()
	 */
	public String toString()
	{
		String chaine="\nOrdinateur";
		
		if(estPortable)
			chaine +=" Portable";
		
		return chaine+super.toString();
	}
}
