/** Parc.java */
package parcInfo;

import java.io.Serializable;
import java.util.*;
import fournisseurs.*;

@SuppressWarnings("serial")

public class Parc implements Serializable
{
	private String nomParc;
	protected TreeMap<String,Equipement>listeEquipement;
	protected TreeMap<String,Fournisseur>listeFournisseur;    
	
	/**
	 * @param nomParc
	 */
	public Parc(String nomParc) {
		this.nomParc = nomParc;
		this.listeEquipement = new TreeMap<String,Equipement> ();
		this.listeFournisseur = new TreeMap<String,Fournisseur>();
	}
	
	/**
	 * @return le nom du Parc
	 */
	public String getNomParc() {
		return nomParc;
	}
	
	/**
	 * @param nomParc  initialise la variable nomParc
	 */
	public void setNomParc(String nomParc) {
		this.nomParc = nomParc;
	}
	
	/**
	 * 	@return listeEquipement:retourne la liste des Equipements
	 */
	public TreeMap<String,Equipement> getListeEquipement(){
		return this.listeEquipement;
	}
	
	/**
	 * 	@return listeFournisseur:retourne la liste des Fournisseurs
	 */
	public TreeMap<String,Fournisseur> getListeFournisseur(){
		return this.listeFournisseur;
	}
	
	
	/**
	 * ajoute un equipement au parc
	 * Cle sur le nom de l'equipement
	 * @param e Equipement
	 */
	public void addEquipement(Equipement e)
	{
		listeEquipement.put(e.getNom(),e);
	}
	
	/**
	 * Retire un Equipement suivant son nom
	 * Si il s'agit d'un ordinateur, on supprime
	 * tous ses composants
	 * @param code
	 */
	public void delEquipement(String code)
	{
		Equipement e;
		e=listeEquipement.get(code) ;
		// Si l'equipement est un ordinateur
		// On supprime tous ses composants
		if(e instanceof Ordinateur){
			((Ordinateur)(e)).delAllComposant();
		}
		listeEquipement.remove(code);
	}
	
	/**
	 * Ajoute un fournisseur au parc
	 * Cle sur le nom du fournisseur
	 * @param f Fournisseur a ajouter
	 */
	public void addFournisseur(Fournisseur f){
		listeFournisseur.put(f.getNom(),f);
	}
	
	/**
	 * Supprime le fournisseur et tous ses catalogues
	 * @param f fournisseur
	 */
	public void delFournisseur(Fournisseur f){
		try{
			f.delAllCatalogue();
			listeFournisseur.remove(f);
		}
		catch(ClassCastException e){};
	}
	
	/**
	 * Calcule le nombre d'impression pour tout le parc
	 * @return entier Nombre de pages Imprimees pour tout le parc
	 * @since 1.5
	 */
	public int getNbPagesImprimeesParc()
	{
		int nbPages=0;
		Equipement e;
		
		for(String code : listeEquipement.keySet()){
			e=listeEquipement.get(code) ;
			if(e instanceof Imprimante){
				nbPages+= ((Imprimante)e).getNbPagesImprimees();
			}
		}
		return nbPages;
	}
	
	/**
	 * Calcule le nombre de connexions pour tout le parc
	 * @return entier Nombre de connexions pour tout le parc
	 * @since 1.5
	 */
	public int getNbConnexionsParc()
	{
		int nbConnexion=0;
		Equipement e;
		for(String code :listeEquipement.keySet()){
			e=listeEquipement.get(code) ;
			if(e instanceof Ordinateur ){
				nbConnexion+=((Ordinateur)e).getNbConnexions();
			}
		}
		return nbConnexion;
	}
	
	
	/**
	 * Compte le nombre d'elements defectueux du parc
	 * Utilise pour la creation de Jtables
	 * @return entier nombre d'equipement defectueux
	 * @since 1.5
	 */
	public int compterEquipementDefectueux()
	{
		int nbDefectueux=0;
		Equipement e;
		Ordinateur o;
		Imprimante i;
		
		for(String code :listeEquipement.keySet()){
			e=listeEquipement.get(code);
			if (e instanceof Ordinateur)
			{
				o=(Ordinateur)e;
				if(o.CalculerEtatOrdinateur())
					nbDefectueux++;
			}
			if (e instanceof Imprimante){
				i=(Imprimante)e;
				if (i.isEtat())
					nbDefectueux++;
			}
		}
		return nbDefectueux;
	}
	
	/**
	 * Liste les equipements Defectueux du parc
	 * @since 1.5
	 */
	public void listerDefectueux()
	{
		Equipement e;
		Ordinateur o;
		Imprimante i;
		
		for(String code :listeEquipement.keySet()){
			e=listeEquipement.get(code);
			if (e instanceof Ordinateur)
			{
				o=(Ordinateur)e;
				if(o.CalculerEtatOrdinateur()){
					// Si c'est un ordinateur on affiche ses composants defectueux
					System.out.println(o);
					System.out.println("Liste des Composants defectueux de l'ordinateur "+o.getNom());
					System.out.print(o.listerComposantsDefectueux());
					System.out.println("\n*******************************************************************************");
				}
			}
			if (e instanceof Imprimante){
				i=(Imprimante)e;
				if (i.isEtat()){
					System.out.println(i);
					System.out.println("\n*******************************************************************************");
				}
			}
			
		}
	}
	
	
	/**
	 * Liste un equipement Defectueux donne
	 * @param code Nom de l'equipement
	 */
	public void listerDefectueux(String code)
	{
		Equipement e;
		Ordinateur o;
		Imprimante i;
		
		e=listeEquipement.get(code);
		if (e instanceof Ordinateur)
		{
			o=(Ordinateur)e;
			if(o.CalculerEtatOrdinateur()){
				System.out.println(o);
				System.out.println("\nListe des Composants defectueux de l'ordinateur "+o.getNom());
				System.out.println(o.listerComposantsDefectueux());
			}
		}
		else
		{
			if (e instanceof Imprimante){
				i=(Imprimante)e;
				if (i.isEtat())
					System.out.println(i);
			}
			else
				System.out.println("Equipement non trouve");
		}
	}
	
	/**
	 * Liste tous les equipements du parc
	 * Pour les ordinateurs on n'affiche que la description
	 * Mais pas ses composants 
	 */
	public void listerEquipement()
	{    	
		for(String code :listeEquipement.keySet()){
			System.out.println(listeEquipement.get(code));
		}
	}
	
	/**
	 * Liste i, equipement en particulier
	 * @param code Nom de l'equipement
	 */
	public void listerEquipement(String code)
	{    	
		System.out.println(listeEquipement.get(code));
	}
	
	/**
	 * Retourne l'equipement decrit par son nom
	 * @param code : Nom de l'equipement
	 * @return Equipement 
	 */
	public Equipement getEquipement(String code)
	{
		return listeEquipement.get(code);
	}
	
	/**
	 * Retourne le fournisseur decrit par son nom
	 * @param code : Nom du fournisseur
	 * @return Fournisseur
	 */
	public Fournisseur getFournisseur(String code)
	{
		return listeFournisseur.get(code);
	}
	
	/**
	 * Calcule pour tous les fournisseurs le meilleur prix
	 * d'un composant donne
	 * @param c : Composants
	 * @return float : Meilleur prix trouve -1 si non trouve
	 * @since 1.5
	 */
	public float getMeilleurPrixFournisseurs(Composants c)
	{
		
		float meilleurPrix=10000000;
		float prix;
		
		for(String code : listeFournisseur.keySet()){
			prix=listeFournisseur.get(code).getMeilleurPrix(c);
			if(prix!=-1 && prix<meilleurPrix)
				meilleurPrix=prix;
		}
		if(meilleurPrix!=10000000)
			return meilleurPrix;
		
		return -1;
	}
	
	/**
	 * Calcule pour tous les fournisseurs le meilleur prix
	 * d'un equipement donne
	 * @param e : Equipement
	 * @return float : Meilleur prix trouve -1 si non trouve
	 * @since 1.5
	 */
	
	public float getMeilleurPrixFournisseurs(Equipement e)
	{
		float meilleurPrix=10000000;
		float prix;
		for(String code : listeFournisseur.keySet()){
			prix=listeFournisseur.get(code).getMeilleurPrix(e);
			if(prix!=-1 && prix<meilleurPrix)
				meilleurPrix=prix;
		}
		if(meilleurPrix!=10000000)
			return meilleurPrix;
		return -1;
		
	}
}
